// USB_Dewpoint v2
#define V2
#define READINGS_UPDATE 10

#include "DHT.h"
#define DHTPIN 2     // what digital pin we're connected to
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
DHT dht(DHTPIN, DHTTYPE);

int epoch = 0;
float temp_amb = 0;
float temp_ch1 = temp_amb;
float temp_ch2 = temp_amb;
float rh = 0;
byte output_ch1 = 0;
byte output_ch2 = output_ch1;
byte output_ch3 = output_ch1;
byte cal_amb = 0;
byte cal_ch1 = cal_amb;
byte cal_ch2 = cal_ch1;
byte threshold_ch1 = 0;
byte threshold_ch2 = threshold_ch1;
bool auto_mode = 1;
bool ch2_3_linked = 1;
byte aggressivity = 1;

#define CH1DEW 9 
#define ledPIN 10  

void setup() {  
  pinMode(CH1DEW, OUTPUT);   // pwm outputs for dew straps
  pinMode(ledPIN, OUTPUT);   // pwm outputs for dew straps
  analogWrite(CH1DEW, 0);
  analogWrite(ledPIN, 0);
  Serial.begin(9600);
  Serial.setTimeout(2000);
  dht.begin();
}

void loop() {
  temp_amb = dht.readTemperature();
  rh = dht.readHumidity();
  float dew_point = dht.computeHeatIndex(temp_amb, rh, false);
  /* In automatic mode the target of USB_Dewpoint is to keep over the night :
      - "Temp1 + Calibration1 > Dewpoint + Threshold1"
      - "Temp2 + Calibration2 > Dewpoint + Threshold2"
     To do so, “Temp1” and “Output1” are linked, “Temp2” and “Output2” are linked.
  */
    
  if (auto_mode) {  
    if ((temp_amb + cal_amb) >= (dew_point + 5.0 + threshold_ch1)) {
      output_ch1 = 0;          
    } 
    else if ((temp_amb + cal_amb) >= (dew_point + 3.0 + threshold_ch1)) {
      output_ch1 = 5;    
    } 
    else if ((temp_amb + cal_amb) >= (dew_point + 2.5 + threshold_ch1)) {
      output_ch1 = 25;
    } 
    else if ((temp_amb + cal_amb) >= (dew_point + 2.0 + threshold_ch1)) {
      output_ch1 = 45;
    } 
    else if ((temp_amb + cal_amb) >= (dew_point + 1.5 + threshold_ch1)) {
      output_ch1 = 65;
    } 
    else if ((temp_amb + cal_amb) >= (dew_point + 1.0 + threshold_ch1)) {
      output_ch1 = 85;
    } 
    else {
      output_ch1 = 100;
    }  
    output_ch2 = output_ch1;
    output_ch3 = output_ch1; 
    analogWrite(CH1DEW, output_ch1 * 2.54 ); // set the PWM value to be 0-254
    analogWrite(ledPIN, output_ch1); // set the PWM value     
  } 
    
  char cmd[7];
  Serial.readBytes(cmd, 6);
  cmd[6] = 0;
  char response[80];
  String command = String(cmd);
  //========================================================== SWHOIS
  if (command.equals("SWHOIS")) {
    Serial.println("UDP2(20210110)");
  //========================================================== SEERAZ
  } else if (command.equals("SEERAZ")) {
    output_ch1 = 0;
    output_ch2 = output_ch1;
    output_ch3 = output_ch1;
    cal_amb = 0;
    cal_ch1 = cal_amb;
    cal_ch2 = cal_amb;    
    threshold_ch1 = 0;
    threshold_ch2 = threshold_ch1;
    auto_mode = 1;
    ch2_3_linked = 1;
    aggressivity = 1;
    Serial.println("EEPROM RESET");
  //========================================================== SGETAL
  } else if (command.equals("SGETAL")) {
    Serial.print("##");
    Serial.print(temp_amb);
    Serial.print("/");
    Serial.print(temp_amb);
    Serial.print("/");
    Serial.print(temp_amb);
    Serial.print("/");
    Serial.print(rh);
    Serial.print("/");
    Serial.print(dew_point);
    Serial.print("/");
    Serial.print(output_ch1);
    Serial.print("/");
    Serial.print(output_ch1);
    Serial.print("/");
    Serial.print(output_ch1);
    Serial.print("/");
    Serial.print(cal_amb);
    Serial.print("/");
    Serial.print(cal_amb);
    Serial.print("/");
    Serial.print(cal_amb);
    Serial.print("/");
    Serial.print(threshold_ch1);
    Serial.print("/");
    Serial.print(threshold_ch1);
    Serial.print("/");
    Serial.print(auto_mode);
    Serial.print("/");
    Serial.print(ch2_3_linked);
    Serial.print("/");
    Serial.print(aggressivity);
    Serial.println("**");
  //========================================================== SCA
  } else if (command.startsWith("SCA")) {   
    cal_ch1 = command.substring(3, 4).toInt();
    cal_ch2 = command.substring(3, 4).toInt();
    cal_amb = command.substring(3, 4).toInt();
    Serial.println("DONE");
  //========================================================== STHR
  } else if (command.startsWith("STHR")) {
    threshold_ch1 = command.substring(4, 5).toInt();
    threshold_ch2 = command.substring(4, 5).toInt();
    Serial.println("DONE");
  //========================================================== SLINK1
  } else if (command.equals("SLINK1")) {
    ch2_3_linked = command.charAt(5) == '1';
    Serial.println("DONE");
  //========================================================== SLINK0
  } else if (command.equals("SLINK0")) {
    ch2_3_linked = 1;
    Serial.println("DONE");
  //========================================================== SAUTO
  } else if (command.startsWith("SAUTO")) {
    auto_mode = command.charAt(5) == '1';
    Serial.println("DONE");
  //========================================================== SAGGR
  } else if (command.startsWith("SAGGR")) {
    aggressivity = command.substring(5, 6).toInt();
    Serial.println("DONE");
  //========================================================== S1O
  } else if (command.startsWith("S1O")) {
    output_ch1 = command.substring(3, 6).toInt();
    output_ch2 = output_ch1;
    output_ch3 = output_ch1;
    analogWrite(CH1DEW, output_ch1 * 2.54 ); // set the PWM value to be 0-254
    analogWrite(ledPIN, output_ch1); // set the PWM value     
    Serial.println("DONE");
  //========================================================== S2O
  } 
  else if (command.startsWith("S2O")) {
    Serial.println("DONE");
  //========================================================== S3O
  } else if (command.startsWith("S3O")) {
    Serial.println("DONE");
  } 
}
